﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using MVCBlog.Core.Repository;

namespace MVCBlog.Core.Database
{
    /// <summary>
    /// Implementation of the <see cref="MVCBlog.Core.Repository.IRepository{T}"/> using the Entity Framework.
    /// </summary>
    /// <typeparam name="T">The <see cref="MVCBlog.Core.Entities.EntityBase"/>.</typeparam>
    internal class EFRepository<T> : IRepository<T> where T : MVCBlog.Core.Entities.EntityBase
    {
        /// <summary>
        /// The context.
        /// </summary>
        private DbContext context;

        /// <summary>
        /// Initializes a new instance of the <see cref="EFRepository&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="context">The context.</param>
        public EFRepository(DbContext context)
        {
            this.context = context;
        }

        /// <summary>
        /// Gets an <see cref="IQueryable{T}"/> to perform further operations.
        /// </summary>
        /// <returns>An <see cref="IQueryable{T}"/>.</returns>
        public IQueryable<T> GetQuery()
        {
            return this.context.Set<T>();
        }

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements.
        /// </summary>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements.</returns>
        public IEnumerable<T> GetAll()
        {
            return this.context.Set<T>().ToList();
        }

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements matching the given condition.
        /// </summary>
        /// <param name="predicate">The condition.</param>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements matching the given condition.</returns>
        public IEnumerable<T> Find(Func<T, bool> predicate)
        {
            return this.context.Set<T>().Where<T>(predicate);
        }

        /// <summary>
        /// Gets a single instance matching the given condition.
        /// </summary>
        /// <param name="predicate">The condition.</param>
        /// <returns>A single instance matching the given condition.</returns>
        public T Single(Func<T, bool> predicate)
        {
            return this.context.Set<T>().Single<T>(predicate);
        }

        /// <summary>
        /// Gets a single instance matching the given condition or null if condition does not match.
        /// </summary>
        /// <param name="predicate">The condition.</param>
        /// <returns>A single instance matching the given condition or <c>null</c> if condition does not match.</returns>
        public T SingleOrDefault(Func<T, bool> predicate)
        {
            return this.context.Set<T>().SingleOrDefault<T>(predicate);
        }

        /// <summary>
        /// Deletes the given entity.
        /// </summary>
        /// <param name="entity">The entity to delete.</param>
        public void Delete(T entity)
        {
            this.context.Set<T>().Remove(entity);
        }

        /// <summary>
        /// Adds the given entity.
        /// </summary>
        /// <param name="entity">The entity to add.</param>
        public void Add(T entity)
        {
            this.context.Set<T>().Add(entity);
        }

        /// <summary>
        /// Attaches the the given entity to the current context.
        /// </summary>
        /// <param name="entity">The entity to attach.</param>
        public void Attach(T entity)
        {
            if (this.context.Entry(entity).State == System.Data.EntityState.Detached)
            {
                this.context.Set<T>().Attach(entity);
            }
        }
    }
}
