﻿using System;
using System.Collections.Generic;
using System.Linq;
using MVCBlog.Core.Repository;
using Palmmedia.Common.Linq;

namespace MVCBlog.Core.Service
{
    /// <summary>
    /// Base class for all services.
    /// </summary>
    /// <typeparam name="T">Class that derives from <see cref="MVCBlog.Core.Entities.EntityBase"/>.</typeparam>
    public abstract class GenericEntityService<T> : IGenericEntityService<T> where T : MVCBlog.Core.Entities.EntityBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GenericEntityService&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="repository">The <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.</param>
        /// <param name="unitOfWork">The <see cref="IUnitOfWork"/>.</param>
        protected GenericEntityService(IRepository<T> repository, IUnitOfWork unitOfWork)
        {
            this.Repository = repository;
            this.UnitOfWork = unitOfWork;
        }

        /// <summary>
        /// Gets or sets the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        protected IRepository<T> Repository { get; set; }

        /// <summary>
        /// Gets or sets the <see cref="IUnitOfWork"/>.
        /// </summary>
        protected IUnitOfWork UnitOfWork { get; set; }

        /// <summary>
        /// Adds the given element to the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to add.</param>
        public virtual void Add(T entity)
        {
            this.Add(entity, true);
        }

        /// <summary>
        /// Adds the given element to the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to add.</param>
        /// <param name="saveChanges">Determines wheather changes should be saved.</param>
        internal void Add(T entity, bool saveChanges)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("entity");
            }

            this.Repository.Add(entity);
            if (saveChanges)
            {
                this.UnitOfWork.SaveChanges();
            }
        }

        /// <summary>
        /// Deletes the given element from the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to delete.</param>
        public virtual void Delete(T entity)
        {
            this.Delete(entity, true);
        }

        /// <summary>
        /// Deletes the given element from the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to delete.</param>
        /// <param name="saveChanges">Determines wheather changes should be saved.</param>
        internal void Delete(T entity, bool saveChanges)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("entity");
            }

            this.Repository.Delete(entity);
            if (saveChanges)
            {
                this.UnitOfWork.SaveChanges();
            }
        }

        /// <summary>
        /// Updates the given element in the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to update.</param>
        public virtual void Update(T entity)
        {
            this.Update(entity, true);
        }

        /// <summary>
        /// Updates the given element in the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to update.</param>
        /// <param name="saveChanges">Determines wheather changes should be saved.</param>
        internal void Update(T entity, bool saveChanges)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("entity");
            }

            this.Repository.Attach(entity);

            if (saveChanges)
            {
                this.UnitOfWork.SaveChanges();
            }
        }

        /// <summary>
        /// Returns the <see cref="MVCBlog.Core.Entities.EntityBase"/> with the given id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>The <see cref="MVCBlog.Core.Entities.EntityBase"/> with the given id.</returns>
        public virtual T GetById(Guid id)
        {
            var entity = this.Repository.SingleOrDefault(e => e.Id == id);
            return entity;
        }

        /// <summary>
        /// Returns an <see cref="IEnumerable{T}"/> from the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <returns>
        /// The <see cref="IEnumerable{T}"/> of all elements.
        /// </returns>
        public virtual IEnumerable<T> GetAll()
        {
            return this.Repository.GetAll();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerable{T}"/> from the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="paging">The <see cref="Paging"/>.</param>
        /// <returns>
        /// The <see cref="PagedResult{T}"/> of all elements.
        /// </returns>
        public virtual PagedResult<T> GetAll(Paging paging)
        {
            var query = this.Repository.GetQuery();
            int totalNumberOfElements = query.Count();

            return new PagedResult<T>(query.SortAndPage(paging), totalNumberOfElements, paging);
        }
    }
}
