﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Web;
using log4net;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Repository;
using Palmmedia.Common;

namespace MVCBlog.Core.Service
{
    /// <summary>
    /// Access to <see cref="BlogEntryComment">BlogEntryComments</see>.
    /// </summary>
    public class BlogEntryCommentService : GenericEntityService<BlogEntryComment>, IBlogEntryCommentService
    {
        /// <summary>
        /// Logger instance.
        /// </summary>
        private static readonly ILog logger = LogManager.GetLogger(typeof(BlogEntryCommentService));

        /// <summary>
        /// Initializes a new instance of the <see cref="BlogEntryCommentService"/> class.
        /// </summary>
        /// <param name="unitOfWork">The <see cref="IUnitOfWork"/>.</param>
        public BlogEntryCommentService(IUnitOfWork unitOfWork)
            : base(unitOfWork.BlogEntryComments, unitOfWork)
        {
        }

        /// <summary>
        /// Determines whether a pingback with the given url exists.
        /// </summary>
        /// <param name="blogEntryId">The id of the <see cref="BlogEntry"/>.</param>
        /// <param name="url">The url.</param>
        /// <returns>True if pingback exists otherwise false.</returns>
        public bool DoesPingbackExist(Guid blogEntryId, string url)
        {
            return this.Repository.GetQuery()
                .Count(c => c.Pingback && c.Comment.Equals(url) && c.BlogEntry.Id == blogEntryId) > 0;
        }

        /// <summary>
        /// Adds the given element to the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to add.</param>
        public override void Add(BlogEntryComment entity)
        {
            base.Add(entity);

            string subject = ConfigurationManager.AppSettings["CommentNotificationSubject"];
            string email = ConfigurationManager.AppSettings["adminEmail"];

            if (string.IsNullOrEmpty(subject) || string.IsNullOrEmpty(email))
            {
                return;
            }

            var body = new StringBuilder();
            body.Append("IP: ");
            body.Append(HttpContext.Current.Request.UserHostAddress);
            body.Append("\nReferrer: ");
            body.Append(HttpContext.Current.Request.UrlReferrer);
            body.Append("\n\nName: ");
            body.Append(entity.Name);
            body.Append("\nEmail: ");
            body.Append(entity.Email);
            body.Append("\nHomepage: ");
            body.Append(entity.Homepage);
            body.Append("\nComment: ");
            body.Append(entity.Comment);

            try
            {
                EmailHelper.SendEmail(
                    email,
                    email,
                    entity.Email,
                    subject,
                    body.ToString());
            }
            catch (EmailException ex)
            {
                logger.Error("Sending email failed.", ex);
            }
        }

        /// <summary>
        /// Returns an <see cref="IEnumerable{T}"/> from the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <returns>
        /// The <see cref="IEnumerable{T}"/> of all elements.
        /// </returns>
        public override IEnumerable<BlogEntryComment> GetAll()
        {
            return this.Repository.GetQuery()
                .OrderByDescending(c => c.Created);
        }
    }
}