﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data.Entity;
using System.Linq;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Repository;
using Palmmedia.Common.Linq;

namespace MVCBlog.Core.Service
{
    /// <summary>
    /// Access to <see cref="BlogEntry">BlogEntries</see>.
    /// </summary>
    public class BlogEntryService : GenericEntityService<BlogEntry>, IBlogEntryService
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="BlogEntryService"/> class.
        /// </summary>
        /// <param name="unitOfWork">The <see cref="IUnitOfWork"/>.</param>
        public BlogEntryService(IUnitOfWork unitOfWork)
            : base(unitOfWork.BlogEntries, unitOfWork)
        {
        }

        /// <summary>
        /// Returns the <see cref="BlogEntry"/> with the given Id.
        /// </summary>
        /// <param name="id">The Id.</param>
        /// <returns>
        /// The <see cref="BlogEntry"/> with the given Id.
        /// </returns>
        public override BlogEntry GetById(Guid id)
        {
            var entry = this.Repository.GetQuery()
                .Include(b => b.Tags)
                .Include(b => b.BlogEntryFiles)
                .Include(b => b.BlogEntryComments)
                .SingleOrDefault(e => e.Id == id);

            return entry;
        }

        /// <summary>
        /// Returns the <see cref="BlogEntry"/> with the given header.
        /// </summary>
        /// <param name="includeInvisibleEntries">Determines whether invisible <see cref="BlogEntry">BlogEntries</see> should be included.</param>
        /// <param name="header">The header.</param>
        /// <returns>
        /// The <see cref="BlogEntry"/> with the given header.
        /// </returns>
        public BlogEntry GetByHeader(bool includeInvisibleEntries, string header)
        {
            var entry = this.Repository.GetQuery()
                    .Include(b => b.Tags)
                    .Include(b => b.BlogEntryFiles)
                    .Include(b => b.BlogEntryComments)
                    .SingleOrDefault(e => (e.Visible || includeInvisibleEntries) && e.HeaderUrl.Equals(header));

            return entry;
        }

        /// <summary>
        /// Returns all <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <param name="includeInvisibleEntries">Determines whether invisible <see cref="BlogEntry">BlogEntries</see> should be included.</param>
        /// <param name="tag">The text of the tag.</param>
        /// <param name="search">The search.</param>
        /// <param name="paging">The <see cref="Paging"/>.</param>
        /// <returns>
        /// All <see cref="BlogEntry">BlogEntries</see>.
        /// </returns>
        public PagedResult<BlogEntry> GetAll(bool includeInvisibleEntries, string tag, string search, Paging paging)
        {
            var query = this.Repository.GetQuery();

            if (!includeInvisibleEntries)
            {
                query = query.Where(e => e.Visible);
            }

            if (!string.IsNullOrEmpty(tag))
            {
                query = query.Where(e => e.Tags.Count(t => t.Name.Equals(tag, StringComparison.OrdinalIgnoreCase)) > 0);
            }

            if (!string.IsNullOrEmpty(search))
            {
                foreach (var item in search.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries))
                {
                    query = query.Where(e => e.Header.Contains(item));
                }
            }

            int totalNumberOfElements = query.Count();
            var entries = query.Include(b => b.BlogEntryComments).SortAndPage(paging);

            return new PagedResult<BlogEntry>(entries, totalNumberOfElements, paging);
        }

        /// <summary>
        /// Returns an <see cref="IEnumerable{T}"/> from the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <returns>
        /// The <see cref="IEnumerable{T}"/> of all elements.
        /// </returns>
        public override IEnumerable<BlogEntry> GetAll()
        {
            var entries = this.Repository.GetQuery()
                .Where(e => e.Visible)
                .Include(b => b.Tags)
                .OrderByDescending(e => e.Created);

            return entries;
        }

        /// <summary>
        /// Returns related <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <param name="entry">The <see cref="BlogEntry"/>.</param>
        /// <param name="numberOfEntries">Maximum number of <see cref="BlogEntry">BlogEntries</see>.</param>
        /// <returns>
        /// The related <see cref="BlogEntry">BlogEntries</see>.
        /// </returns>
        public IEnumerable<BlogEntry> GetRelatedBlogEntries(BlogEntry entry, int numberOfEntries)
        {
            if (entry == null)
            {
                throw new ArgumentNullException("entry");
            }

            var tagIds = entry.Tags.Select(t => t.Id).ToArray();

            var query = this.Repository.GetQuery()
                .Where(e => e.Visible && e.Id != entry.Id)
                .Where(e => e.Tags.Any(t => tagIds.Contains(t.Id)))
                .OrderByDescending(e => e.Tags.Count(t => tagIds.Contains(t.Id)))
                .ThenByDescending(e => e.Created)
                .Take(numberOfEntries);

            return query;
        }

        /// <summary>
        /// Adds the given <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="entry">The <see cref="BlogEntry"/> to add.</param>
        /// <param name="tags">The tags.</param>
        public void Add(BlogEntry entry, IEnumerable<string> tags)
        {
            if (entry == null)
            {
                throw new ArgumentNullException("entry");
            }

            if (tags == null)
            {
                throw new ArgumentNullException("tags");
            }

            this.AddTags(entry, tags);
            base.Add(entry);
        }

        /// <summary>
        /// Updates the given <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="entry">The <see cref="BlogEntry"/> to update.</param>
        /// <param name="tags">The tags.</param>
        public void Update(BlogEntry entry, IEnumerable<string> tags)
        {
            if (entry == null)
            {
                throw new ArgumentNullException("entry");
            }

            if (tags == null)
            {
                throw new ArgumentNullException("tags");
            }

            this.AddTags(entry, tags);
            base.Update(entry);
        }

        /// <summary>
        /// Deletes the given element from the <see cref="MVCBlog.Core.Repository.IRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to delete.</param>
        public override void Delete(BlogEntry entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("entity");
            }

            foreach (var blogEntryFile in entity.BlogEntryFiles)
            {
                blogEntryFile.DeleteData();
            }

            base.Delete(entity);
        }

        /// <summary>
        /// Adds the file to the <see cref="BlogEntry"/> with the given Id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="fileName">The filename.</param>
        /// <param name="file">The file.</param>
        public void AddFile(Guid id, string fileName, byte[] file)
        {
            if (fileName == null)
            {
                throw new ArgumentNullException("fileName");
            }

            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            var entry = this.Repository.SingleOrDefault(e => e.Id == id);

            if (entry != null)
            {
                int indexOfLastDot = fileName.LastIndexOf('.');
                string name = fileName.Substring(0, indexOfLastDot);
                string extension = fileName.Substring(indexOfLastDot + 1, fileName.Length - indexOfLastDot - 1);

                var blogEntryFile = new BlogEntryFile() { BlogEntry = entry, Name = name, Extension = extension };
                blogEntryFile.Data = file;

                entry.BlogEntryFiles.Add(blogEntryFile);
                UnitOfWork.SaveChanges();
            }
        }

        /// <summary>
        /// Adds the tags to the given <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="entry">The entry.</param>
        /// <param name="tags">The tags.</param>
        private void AddTags(BlogEntry entry, IEnumerable<string> tags)
        {
            var existingTags = this.UnitOfWork.Tags.GetAll().ToList();

            if (entry.Tags == null)
            {
                entry.Tags = new Collection<Tag>();
            }

            foreach (var tag in entry.Tags.Where(t => !tags.Contains(t.Name)).ToArray())
            {
                entry.Tags.Remove(tag);
            }

            foreach (var tag in tags.Where(t => !entry.Tags.Select(et => et.Name).Contains(t)).ToArray())
            {
                var existingTag = existingTags.SingleOrDefault(t => t.Name.Equals(tag, StringComparison.OrdinalIgnoreCase));

                if (existingTag == null)
                {
                    existingTag = new Tag() { Name = tag };
                    existingTags.Add(existingTag);
                }

                entry.Tags.Add(existingTag);
            }
        }
    }
}
