using System;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Service;
using MVCBlog.Website.Models.OutputModels.Administration;

namespace MVCBlog.Website.Controllers
{
    /// <summary>
    /// Controller for the administration tasks.
    /// </summary>
    [Authorize()]
    public partial class AdministrationController : Controller
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AdministrationController"/> class.
        /// </summary>
        /// <param name="blogEntryService">The blog entry service.</param>
        /// <param name="imageService">The image service.</param>
        /// <param name="tagService">The tag service.</param>
        /// <param name="blogEntryFileService">The blog entry file service.</param>
        public AdministrationController(
            IBlogEntryService blogEntryService,
            IImageService imageService,
            ITagService tagService,
            IBlogEntryFileService blogEntryFileService)
        {
            this.BlogEntryService = blogEntryService;
            this.ImageService = imageService;
            this.TagService = tagService;
            this.BlogEntryFileService = blogEntryFileService;
        }

        #region Properties

        /// <summary>
        /// Gets or sets the blog entry service.
        /// </summary>
        public IBlogEntryService BlogEntryService { get; set; }

        /// <summary>
        /// Gets or sets the image service.
        /// </summary>
        public IImageService ImageService { get; set; }

        /// <summary>
        /// Gets or sets the tag service.
        /// </summary>
        public ITagService TagService { get; set; }

        /// <summary>
        /// Gets or sets the blog entry file service.
        /// </summary>
        public IBlogEntryFileService BlogEntryFileService { get; set; }

        #endregion

        /// <summary>
        /// Shows an overview.
        /// </summary>
        /// <returns>View showing an overview.</returns>
        public virtual ActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Shows all downloads.
        /// </summary>
        /// <returns>View showing all downloads.</returns>
        public virtual ActionResult Downloads()
        {
            var model = this.BlogEntryFileService.GetAll();

            return View(model);
        }

        /// <summary>
        /// Show a single <see cref="BlogEntry"/> for editing.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        public virtual ActionResult EditBlogEntry(Guid? id)
        {
            var blogEntry = id.HasValue ? this.BlogEntryService.GetById(id.Value) : new BlogEntry();

            return View(new EditBlogEntry() { BlogEntry = blogEntry, Tags = this.TagService.GetAll(), Images = this.ImageService.GetAll(), IsUpdate = id.HasValue });
        }

        /// <summary>
        /// Saves the changes applied to the <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <param name="blogEntry">The <see cref="BlogEntry"/>.</param>
        /// <param name="formValues">The form values.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [HttpPost]
        [ValidateInput(false)]
        public virtual ActionResult EditBlogEntry(Guid? id, [Bind(Include = "Header, Author, ShortContent, Content, Visible")]BlogEntry blogEntry, FormCollection formValues)
        {
            if (!this.ModelState.IsValid)
            {
                return View(new EditBlogEntry() { BlogEntry = blogEntry, Tags = this.TagService.GetAll(), Images = this.ImageService.GetAll(), IsUpdate = id.HasValue });
            }

            if (id.HasValue)
            {
                var existingBlogEntry = this.BlogEntryService.GetById(id.Value);
                existingBlogEntry.Header = blogEntry.Header;
                existingBlogEntry.Author = blogEntry.Author;
                existingBlogEntry.ShortContent = blogEntry.ShortContent;
                existingBlogEntry.Content = blogEntry.Content;
                existingBlogEntry.Visible = blogEntry.Visible;

                blogEntry = existingBlogEntry;
            }

            var tags = formValues.AllKeys.Where(k => k.StartsWith("Tag") && !string.IsNullOrEmpty(formValues[k])).Select(k => formValues[k]);

            if (id.HasValue)
            {
                this.BlogEntryService.Update(blogEntry, tags);
            }
            else
            {
                this.BlogEntryService.Add(blogEntry, tags);
            }

            return this.RedirectToAction(MVC.Administration.EditBlogEntry(blogEntry.Id));
        }

        /// <summary>
        /// Performs all pingback requests. All URLs mentioned in the BlogPost are notified.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <returns>A view showing the results of the pingback requests.</returns>
        [HttpPost]
        public virtual ActionResult PerformPingbacks(Guid id)
        {
            var blogEntry = this.BlogEntryService.GetById(id);

            var pingbackResults = Palmmedia.Common.Net.PingBack.PingBackService.PerformPingBack(
                this.Request.Url.GetLeftPart(UriPartial.Authority) + Url.Action(blogEntry.Url, MVC.Blog.Name),
                blogEntry.Content);

            return View(MVC.Administration.Views.Pingback, pingbackResults);
        }

        /// <summary>
        /// Adds a file to a <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <param name="file">The uploaded file.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [HttpPost]
        public virtual ActionResult FileUpload(Guid id, HttpPostedFileBase file)
        {
            if (file != null && file.ContentLength > 0)
            {
                var content = new byte[file.ContentLength];
                file.InputStream.Read(content, 0, file.ContentLength);
                this.BlogEntryService.AddFile(id, file.FileName, content);
            }

            return this.RedirectToAction(MVC.Administration.EditBlogEntry(id));
        }

        /// <summary>
        /// Files the delete.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <param name="fileId">The file id.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        public virtual ActionResult FileDelete(Guid id, Guid fileId)
        {
            var blogEntryFile = this.BlogEntryFileService.GetById(fileId);

            this.BlogEntryFileService.Delete(blogEntryFile);

            return this.RedirectToAction(MVC.Administration.EditBlogEntry(id));
        }

        /// <summary>
        /// Shows all <see cref="MVCBlog.Core.Entities.Image">Images</see>.
        /// </summary>
        /// <returns>A view showing all <see cref="MVCBlog.Core.Entities.Image">Images</see>.</returns>
        public virtual ActionResult Images()
        {
            var images = this.ImageService.GetAll();
            return View(images);
        }

        /// <summary>
        /// Upload an image
        /// </summary>
        /// <param name="file">The uploaded file.</param>
        /// <returns>A view showing all <see cref="MVCBlog.Core.Entities.Image">Images</see>.</returns>
        [HttpPost]
        public virtual ActionResult ImageUpload(HttpPostedFileBase file)
        {
            if (file != null && file.ContentLength > 0)
            {
                var content = new byte[file.ContentLength];
                file.InputStream.Read(content, 0, file.ContentLength);
                this.ImageService.Add(file.FileName, content);
            }

            return this.RedirectToAction(MVC.Administration.Images());
        }
    }
}
