using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.ServiceModel.Syndication;
using System.Web.Mvc;
using System.Web.UI.WebControls;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Service;
using MVCBlog.Website.Models.OutputModels.Blog;
using MvcSiteMapProvider.Filters;
using Palmmedia.Common.Linq;
using Palmmedia.Common.Net.Mvc;

namespace MVCBlog.Website.Controllers
{
    /// <summary>
    /// Controller for the blog.
    /// </summary>
    public partial class BlogController : Controller
    {
        /// <summary>
        /// Number of <see cref="BlogEntry">BlogEntries</see> per page.
        /// </summary>
        private const int ENTRIESPERPAGE = 8;

        /// <summary>
        /// Initializes a new instance of the <see cref="BlogController"/> class.
        /// </summary>
        /// <param name="blogEntryService">The blog entry service.</param>
        /// <param name="imageService">The image service.</param>
        /// <param name="tagService">The tag service.</param>
        /// <param name="blogEntryCommentService">The blog entry comment service.</param>
        /// <param name="blogEntryFileService">The blog entry file service.</param>
        public BlogController(
            IBlogEntryService blogEntryService,
            IImageService imageService,
            ITagService tagService,
            IBlogEntryCommentService blogEntryCommentService,
            IBlogEntryFileService blogEntryFileService)
        {
            this.BlogEntryService = blogEntryService;
            this.TagService = tagService;
            this.BlogEntryCommentService = blogEntryCommentService;
            this.BlogEntryFileService = blogEntryFileService;
        }

        #region Properties

        /// <summary>
        /// Gets or sets the blog entry service.
        /// </summary>
        public IBlogEntryService BlogEntryService { get; set; }

        /// <summary>
        /// Gets or sets the tag service.
        /// </summary>
        public ITagService TagService { get; set; }

        /// <summary>
        /// Gets or sets the blog entry comment service.
        /// </summary>
        public IBlogEntryCommentService BlogEntryCommentService { get; set; }

        /// <summary>
        /// Gets or sets the blog entry file service.
        /// </summary>
        public IBlogEntryFileService BlogEntryFileService { get; set; }

        #endregion

        /// <summary>
        /// Shows all <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <param name="tag">The tag.</param>
        /// <param name="search">The search.</param>
        /// <param name="page">Number of the page.</param>
        /// <returns>A view showing all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [ValidateInput(false)]
        public virtual ActionResult Index(string tag, string search, int? page)
        {
            var entries = this.BlogEntryService.GetAll(
                this.Request.IsAuthenticated,
                tag,
                search,
                new Paging(page.GetValueOrDefault(1), ENTRIESPERPAGE, PropertyResolver.GetPropertyName<BlogEntry>(b => b.Created), SortDirection.Descending));

            var model = new IndexModel();
            model.Entries = entries.ToArray();
            model.TotalPages = (int)Math.Ceiling((double)entries.TotalNumberOfItems / ENTRIESPERPAGE);
            model.CurrentPage = page;
            model.Tag = tag;
            model.Search = search;

            return View(model);
        }

        /// <summary>
        /// Shows a single <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [SiteMapTitle("Header")]
        public virtual ActionResult Entry(string id)
        {
            var entry = this.BlogEntryService.GetByHeader(this.Request.IsAuthenticated, id);

            if (entry == null)
            {
                return new HttpNotFoundWithViewResult(MVC.Shared.Views.NotFound);
            }

            return View(new BlogEntryDetail()
            {
                BlogEntry = entry,
                RelatedBlogEntries = this.BlogEntryService.GetRelatedBlogEntries(entry, 3).ToArray()
            });
        }

        /// <summary>
        /// Adds a <see cref="Comment"/> to a <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="blogEntryComment">The comment.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [SiteMapTitle("Header")]
        [Palmmedia.Common.Net.Mvc.ReferrerAuthorize()]
        [Palmmedia.Common.Net.Mvc.SpamProtection(4)]
        [ValidateAntiForgeryToken()]
        [HttpPost]
        public virtual ActionResult Entry(string id, [Bind(Include = "Name, Email, Homepage, Comment")] BlogEntryComment blogEntryComment)
        {
            var entry = this.BlogEntryService.GetByHeader(this.Request.IsAuthenticated, id);

            if (entry == null)
            {
                return new HttpNotFoundResult();
            }

            if (!ModelState.IsValid)
            {
                var errorModel = new BlogEntryDetail()
                {
                    BlogEntry = entry
                };

                if (this.Request.IsAjaxRequest())
                {
                    return PartialView(MVC.Blog.Views._CommentsControl, errorModel);
                }
                else
                {
                    errorModel.RelatedBlogEntries = this.BlogEntryService.GetRelatedBlogEntries(entry, 3).ToArray();
                    return View(errorModel);
                }
            }

            blogEntryComment.AdminPost = this.Request.IsAuthenticated;
            blogEntryComment.BlogEntryId = entry.Id;
            entry.BlogEntryComments.Add(blogEntryComment);

            this.BlogEntryCommentService.Add(blogEntryComment);

            var model = new BlogEntryDetail()
            {
                BlogEntry = entry,
                HideNewCommentsForm = true
            };

            if (this.Request.IsAjaxRequest())
            {
                return PartialView(MVC.Blog.Views._CommentsControl, model);
            }
            else
            {
                model.RelatedBlogEntries = this.BlogEntryService.GetRelatedBlogEntries(entry, 3).ToArray();
                return View(model);
            }
        }

        /// <summary>
        /// Shows a tagcloud.
        /// </summary>
        /// <returns>A view showing a tagcloud.</returns>
        [ChildActionOnly]
        [OutputCache(Duration = 3600)]
        public virtual ActionResult Tags()
        {
            var tags = this.TagService.GetAll();

            if (tags.Any())
            {
                return PartialView(MVC.Shared.Views._TagListControl, tags);
            }
            else
            {
                return new EmptyResult();
            }
        }

        /// <summary>
        /// Returns an RSS-Feed of all <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <returns>RSS-Feed of all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [OutputCache(CacheProfile = "Long")]
        public virtual ActionResult Feed()
        {
            var entries = this.BlogEntryService.GetAll();

            string baseUrl = this.Request.Url.GetLeftPart(UriPartial.Authority);

            var feed = new SyndicationFeed(
                ConfigurationManager.AppSettings["BlogName"],
                ConfigurationManager.AppSettings["BlogDescription"],
                new Uri(this.Request.Url.GetLeftPart(UriPartial.Authority) + Url.Action(MVC.Blog.Feed())));

            feed.BaseUri = new Uri(baseUrl);

            if (entries.Any())
            {
                DateTime lastCreatedDate = entries.ElementAt(0).Created;

                DateTime? lastModifiedDate = entries.OrderByDescending(e => e.Modified).First().Modified;
                if (lastModifiedDate.HasValue && lastModifiedDate.Value > lastCreatedDate)
                {
                    feed.LastUpdatedTime = lastModifiedDate.Value;
                }
                else
                {
                    feed.LastUpdatedTime = lastCreatedDate;
                }
            }

            var feedItems = new List<SyndicationItem>();

            foreach (var blogEntry in entries)
            {
                var syndicationItem = new SyndicationItem(
                    blogEntry.Header,
                    SyndicationContent.CreateXhtmlContent(blogEntry.Content),
                    new Uri(baseUrl + Url.Action(blogEntry.Url)),
                    blogEntry.Id.ToString(),
                    blogEntry.Modified.HasValue ? blogEntry.Modified.Value : blogEntry.Created);

                if (!string.IsNullOrEmpty(blogEntry.Author))
                {
                    syndicationItem.Authors.Add(new SyndicationPerson() { Name = blogEntry.Author });
                }

                foreach (var tag in blogEntry.Tags)
                {
                    syndicationItem.Categories.Add(new SyndicationCategory(tag.Name));
                }

                feedItems.Add(syndicationItem);
            }

            feed.Items = feedItems;

            return new Palmmedia.Common.Net.Mvc.Feed.RssSyndicationFeedActionResult(feed);
        }

        /// <summary>
        /// Deletes the <see cref="BlogEntry"/> with the given id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>A view showing all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [Authorize()]
        public virtual ActionResult Delete(Guid id)
        {
            var blogEntry = this.BlogEntryService.GetById(id);

            this.BlogEntryService.Delete(blogEntry);

            return this.RedirectToAction(MVC.Blog.Index());
        }

        /// <summary>
        /// Deletes the <see cref="Comment"/> with the given id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>A view showing all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [Authorize()]
        public virtual ActionResult DeleteComment(Guid id)
        {
            var blogEntryComment = this.BlogEntryCommentService.GetById(id);

            this.BlogEntryCommentService.Delete(blogEntryComment);

            return this.Redirect(this.Request.UrlReferrer.ToString());
        }

        /// <summary>
        /// Streams the <see cref="BlogEntryFile"/> with the given id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>The <see cref="BlogEntryFile"/> as Download.</returns>
        public virtual ActionResult Download(Guid id)
        {
            var blogEntryFile = this.BlogEntryFileService.GetById(id);

            if (blogEntryFile == null)
            {
                return new HttpNotFoundWithViewResult(MVC.Shared.Views.NotFound);
            }

            blogEntryFile.Counter++;
            this.BlogEntryFileService.Update(blogEntryFile);

            var file = new FileContentResult(blogEntryFile.Data, "application/octet-stream");
            file.FileDownloadName = blogEntryFile.Name + "." + blogEntryFile.Extension;
            return file;
        }

        /// <summary>
        /// Shows the imprint.
        /// </summary>
        /// <returns>The imprint view.</returns>
        public virtual ActionResult Imprint()
        {
            return View();
        }

        /// <summary>
        /// Shows the donate view.
        /// </summary>
        /// <returns>The donate view.</returns>
        public virtual ActionResult Donate()
        {
            return View();
        }
    }
}
